package onig4j;

import static onig4j.OnigRegex.onig_get_default_case_fold_flag;
import static onig4j.OnigRegex.onig_set_default_case_fold_flag;
import static onig4j.OnigRegex.ONIG_NORMAL;

/**
 * OnigCaseFoldType enum class.
 * @author calico
 * @see <a href="http://www.geocities.jp/kosako3/oniguruma/">Oniguruma</a>
 * @see <a href="http://www.geocities.jp/kosako3/oniguruma/index_ja.html">鬼車</a>
 */
public enum OnigCaseFoldType {
    /**
     * case fold type for Turkish and Azerbaijan language.<br/>
     * NOTE: The Oniguruma library where USE_UNICODE_CASE_FOLD_TURKISH_AZERI was enabled is necessary.
     */
    ONIGENC_CASE_FOLD_TURKISH_AZERI(1 << 20),
    /** minimum */
    ONIGENC_CASE_FOLD_MIN(1 << 30);
    
    final int value;
    
    private OnigCaseFoldType(int value) {
        this.value = value;
    }
    
    static OnigCaseFoldType valueOf(final int value) {
        for (final OnigCaseFoldType cf : OnigCaseFoldType.values()) {
            if (value == cf.value) {
                return cf;
            }
        }
        throw new IllegalArgumentException("No enum const class " + OnigCaseFoldType.class.getName() + ".value(" + value + ")");
    }
    
    /**
     * invoke onig_get_default_case_fold_flag() function.
     * @return The default case fold flag
     */
    public static OnigCaseFoldType getDefaultCaseFoldType() {
        return valueOf(onig_get_default_case_fold_flag());
    }
    
    /**
     * invoke onig_set_default_case_fold_flag() function.
     * @param flag The default case fold flag
     */
    public static void setDefaultCaseFoldType(OnigCaseFoldType flag) {
        final int ret = onig_set_default_case_fold_flag(flag.value);
        if (ret != ONIG_NORMAL) {
            throw new IllegalArgumentException(OnigRegex.getErrorMessage(ret, null));
        }
    }

//    public final static int INTERNAL_ONIGENC_CASE_FOLD_MULTI_CHAR = (1 << 30);
    public final static int ONIGENC_MAX_COMP_CASE_FOLD_CODE_LEN = 3;
    public final static int ONIGENC_GET_CASE_FOLD_CODES_MAX_NUM = 13;
}
