package onig4j.java.util.regex;

import java.util.List;
import onig4j.OnigOptionType;
import onig4j.OnigRegex;
import onig4j.OnigSyntaxType;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 *
 * @author calico
 */
public class PatternTest {

    @Test
    public void split() {
        final String regex = ",";
        final String input = "abc,ｄ,eｆg,h,ｉ,,j,,,,ｋ,,,";
        
        final String[] values = java.util.regex.Pattern.compile(regex).split(input);
        for (final String v : values) {
            System.out.println("values :[" + v + "]");
        }
        assertEquals(11, values.length);
        
        Pattern pattern = Pattern.compile(regex);
        final String[] values2 = pattern.split(input);
        assertEquals(values.length, values2.length);
        for (int i = 0; i < values.length; ++i) {
            assertEquals(values[i], values2[i]);
        }
        
        OnigRegex reg = new OnigRegex(regex, OnigSyntaxType.ONIG_SYNTAX_JAVA);
        final List<String> values3 = reg.split(input, 0, OnigOptionType.ONIG_OPTION_NONE);
        for (final String v : values3) {
            System.out.println("values3:[" + v + "]");
        }
        assertEquals(values.length, values3.size());
        for (int i = 0; i < values.length; ++i) {
            assertEquals(values[i], values3.get(i));
        }
    }

    @Test
    public void split2() {
//        final String regex = ",\\p{Alpha}";
        final String regex = ",[a-zA-Z]";
        final String input = "abc,ｄ,eｆg,h,ｉ,,j,,,,ｋ,,,";
        
//        final String[] values
//                = java.util.regex.Pattern.compile(regex).split(input);
        final String[] values
                = java.util.regex.Pattern.compile(regex, java.util.regex.Pattern.UNICODE_CASE).split(input);
        for (final String v : values) {
            System.out.println("values :[" + v + "]");
        }
        assertEquals(4, values.length);
        
        OnigRegex reg = new OnigRegex(regex, OnigSyntaxType.ONIG_SYNTAX_JAVA);
        final List<String> values3 = reg.split(input, 0, OnigOptionType.ONIG_OPTION_NONE);
        for (final String v : values3) {
            System.out.println("values3:[" + v + "]");
        }
        assertEquals(values.length, values3.size());
        for (int i = 0; i < values.length; ++i) {
            assertEquals(values[i], values3.get(i));
        }
         
        Pattern pattern = Pattern.compile(regex);      
        final String[] values2 = pattern.split(input);
        assertEquals(values.length, values2.length);
        for (int i = 0; i < values.length; ++i) {
            assertEquals(values[i], values2[i]);
        }
   }

    @Test
    public void split3() {
//        final int options = java.util.regex.Pattern.MULTILINE;
        final int options = java.util.regex.Pattern.MULTILINE | java.util.regex.Pattern.UNIX_LINES;
        final String regex = "$";
//        final String regex = "(\n|\r\n|\r|\u0085|\u2028|\u2029)";
//        final String regex = "\\b";
        final String input = "あいうえお\nかきくけこ\r\nさしすせそ\rたちつてと\u0085なにぬねの\u2028はひふへほ\u2029まみむめも";
        
        final String[] values
                = java.util.regex.Pattern.compile(regex, options).split(input);
        for (final String v : values) {
            System.out.println("values :[" + v + "]");
        }
        
        Pattern pattern = Pattern.compile(regex, options);      
        final String[] values2 = pattern.split(input, -1);
        for (final String v : values2) {
            System.out.println("values2:[" + v + "]");
        }
//        assertEquals(values.length, values2.length);
        for (int i = 0; i < values.length; ++i) {
            assertEquals(values[i], values2[i]);
        }
        
        OnigRegex reg = new OnigRegex(regex, OnigSyntaxType.ONIG_SYNTAX_JAVA, OnigOptionType.ONIG_OPTION_NEGATE_SINGLELINE);
        final List<String> values3 = reg.split(input, -1, OnigOptionType.ONIG_OPTION_NONE);
        for (final String v : values3) {
            System.out.println("values3:[" + v + "]");
        }
        assertEquals(values2.length, values3.size());
        for (int i = 0; i < values2.length; ++i) {
            assertEquals(values2[i], values3.get(i));
        }
   }

    @Test
    public void split4() {
        final int options = 0;
        final String regex = "\\bdef";
        final String input = "abcdefghijklmn define";
        
        final String[] values
                = java.util.regex.Pattern.compile(regex, options).split(input);
        for (final String v : values) {
            System.out.println("values :[" + v + "]");
        }
        
        Pattern pattern = Pattern.compile(regex, options);      
        final String[] values2 = pattern.split(input, -1);
        for (final String v : values2) {
            System.out.println("values2:[" + v + "]");
        }
//        assertEquals(values.length, values2.length);
        for (int i = 0; i < values.length; ++i) {
            assertEquals(values[i], values2[i]);
        }
        
        OnigRegex reg = new OnigRegex(regex, OnigSyntaxType.ONIG_SYNTAX_JAVA);
        final List<String> values3 = reg.split(input, -1, OnigOptionType.ONIG_OPTION_NONE);
        for (final String v : values3) {
            System.out.println("values3:[" + v + "]");
        }
        assertEquals(values.length, values3.size());
        for (int i = 0; i < values.length; ++i) {
            assertEquals(values[i], values3.get(i));
        }
   }

    @Test
    public void split5() {
//        final int options = java.util.regex.Pattern.MULTILINE;
        final int options = java.util.regex.Pattern.MULTILINE | java.util.regex.Pattern.UNIX_LINES;
        final String regex = "[^$]";
        final String input = "\r\nあいうえお\nかきくけこ\r\nさしすせそ\rたちつてと\u0085なにぬねの\u2028はひふへほ\u2029まみむめも";
        
        final String[] values
                = java.util.regex.Pattern.compile(regex, options).split(input);
        for (final String v : values) {
            System.out.println("values :[" + v + "]");
        }
        
        Pattern pattern = Pattern.compile(regex, options);      
        final String[] values2 = pattern.split(input, -1);
        for (final String v : values2) {
            System.out.println("values2:[" + v + "]");
        }
//        assertEquals(values.length, values2.length);
        for (int i = 0; i < values.length; ++i) {
            assertEquals(values[i], values2[i]);
        }
        
        OnigRegex reg = new OnigRegex(regex, OnigSyntaxType.ONIG_SYNTAX_JAVA, OnigOptionType.ONIG_OPTION_NEGATE_SINGLELINE, OnigOptionType.ONIG_OPTION_FIND_LONGEST);
        final List<String> values3 = reg.split(input, -1, OnigOptionType.ONIG_OPTION_NONE);
        for (final String v : values3) {
            System.out.println("values3:[" + v + "]");
        }
        assertEquals(values2.length, values3.size());
        for (int i = 0; i < values2.length; ++i) {
            assertEquals(values2[i], values3.get(i));
        }
    }
    
    @Test
    public void unicodeEscape() {
        final String regex = "\uuuuuuu3042い\\u3046え\\u304a";
        final String input = "かきくけこあ\u3044うえ\u304Aさしすせそあいう\u3048おたちつてとあいうえおなにぬねの";
     
        final java.util.regex.Matcher matcher = java.util.regex.Pattern.compile(regex).matcher(input);
        assertTrue(matcher.find());
        assertEquals(5, matcher.start());
        assertEquals("あいうえお", matcher.group());
        assertTrue(matcher.find());
        assertEquals(15, matcher.start());
        assertTrue(matcher.find());
        assertEquals(25, matcher.start());
        assertFalse(matcher.find());
        
        final Matcher matcher2 = Pattern.compile(regex).matcher(input);
        assertTrue(matcher2.find());
        assertEquals(5, matcher2.start());
        assertEquals("あいうえお", matcher2.group());
        assertTrue(matcher2.find());
        assertEquals(15, matcher2.start());
        assertTrue(matcher2.find());
        assertEquals(25, matcher2.start());
        assertFalse(matcher2.find());
    }
}